@@ -30,6 +30,13 @@
 GST_DEBUG_CATEGORY_STATIC (gsmdec_debug);
 #define GST_CAT_DEFAULT (gsmdec_debug)
 
+/* elementfactory information */
+static const GstElementDetails gst_gsmdec_details =
+GST_ELEMENT_DETAILS ("GSM audio decoder",
+    "Codec/Decoder/Audio",
+    "Decodes GSM encoded audio",
+    "Philippe Khalaf <burger@speedy.org>");
+
 /* GSMDec signals and args */
 enum
 {
@@ -80,14 +87,12 @@ gst_gsmdec_get_type (void)
   return gsmdec_type;
 }
 
-#define ENCODED_SAMPLES	160
-
 static GstStaticPadTemplate gsmdec_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-gsm, rate = (int) 8000, channels = (int) 1; "
-        "audio/ms-gsm, rate = (int) [1, MAX], channels = (int) 1")
+        "audio/ms-gsm, rate = (int) 8000, channels = (int) 1")
     );
 
 static GstStaticPadTemplate gsmdec_src_template =
@@ -98,7 +103,7 @@ GST_STATIC_PAD_TEMPLATE ("src",
         "endianness = (int) BYTE_ORDER, "
         "signed = (boolean) true, "
         "width = (int) 16, "
-        "depth = (int) 16, " "rate = (int) [1, MAX], " "channels = (int) 1")
+        "depth = (int) 16, " "rate = (int) 8000, " "channels = (int) 1")
     );
 
 static void
@@ -106,21 +111,21 @@ gst_gsmdec_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gsmdec_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gsmdec_src_template);
-  gst_element_class_set_details_simple (element_class, "GSM audio decoder",
-      "Codec/Decoder/Audio",
-      "Decodes GSM encoded audio", "Philippe Khalaf <burger@speedy.org>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gsmdec_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gsmdec_src_template));
+  gst_element_class_set_details (element_class, &gst_gsmdec_details);
 }
 
 static void
 gst_gsmdec_class_init (GstGSMDec * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
@@ -134,14 +139,16 @@ gst_gsmdec_init (GstGSMDec * gsmdec)
 {
   /* create the sink and src pads */
   gsmdec->sinkpad =
-      gst_pad_new_from_static_template (&gsmdec_sink_template, "sink");
+      gst_pad_new_from_template (gst_static_pad_template_get
+      (&gsmdec_sink_template), "sink");
   gst_pad_set_setcaps_function (gsmdec->sinkpad, gst_gsmdec_sink_setcaps);
   gst_pad_set_event_function (gsmdec->sinkpad, gst_gsmdec_sink_event);
   gst_pad_set_chain_function (gsmdec->sinkpad, gst_gsmdec_chain);
   gst_element_add_pad (GST_ELEMENT (gsmdec), gsmdec->sinkpad);
 
   gsmdec->srcpad =
-      gst_pad_new_from_static_template (&gsmdec_src_template, "src");
+      gst_pad_new_from_template (gst_static_pad_template_get
+      (&gsmdec_src_template), "src");
   gst_element_add_pad (GST_ELEMENT (gsmdec), gsmdec->srcpad);
 
   gsmdec->state = gsm_create ();
@@ -170,7 +177,6 @@ gst_gsmdec_sink_setcaps (GstPad * pad, GstCaps * caps)
   GstGSMDec *gsmdec;
   GstCaps *srccaps;
   GstStructure *s;
-  gboolean ret = FALSE;
 
   gsmdec = GST_GSMDEC (gst_pad_get_parent (pad));
 
@@ -186,41 +192,25 @@ gst_gsmdec_sink_setcaps (GstPad * pad, GstCaps * caps)
   else
     goto wrong_caps;
 
-  if (!gst_structure_get_int (s, "rate", &gsmdec->rate)) {
-    GST_WARNING_OBJECT (gsmdec, "missing sample rate parameter from sink caps");
-    goto beach;
-  }
-
   /* MSGSM needs different framing */
   gsm_option (gsmdec->state, GSM_OPT_WAV49, &gsmdec->use_wav49);
 
-  gsmdec->duration = gst_util_uint64_scale (ENCODED_SAMPLES,
-      GST_SECOND, gsmdec->rate);
+  /* we only have one possible source caps, which is the same as our template. */
+  srccaps = gst_static_pad_template_get_caps (&gsmdec_src_template);
 
-  /* Setting up src caps based on the input sample rate. */
-  srccaps = gst_caps_new_simple ("audio/x-raw-int",
-      "endianness", G_TYPE_INT, G_BYTE_ORDER,
-      "signed", G_TYPE_BOOLEAN, TRUE,
-      "width", G_TYPE_INT, 16,
-      "depth", G_TYPE_INT, 16,
-      "rate", G_TYPE_INT, gsmdec->rate, "channels", G_TYPE_INT, 1, NULL);
+  gst_pad_set_caps (gsmdec->srcpad, srccaps);
 
-  ret = gst_pad_set_caps (gsmdec->srcpad, srccaps);
-
-  gst_caps_unref (srccaps);
   gst_object_unref (gsmdec);
 
-  return ret;
+  return TRUE;
 
   /* ERRORS */
 wrong_caps:
-
-  GST_ERROR_OBJECT (gsmdec, "invalid caps received");
-
-beach:
-  gst_object_unref (gsmdec);
-
-  return ret;
+  {
+    GST_ERROR_OBJECT (gsmdec, "invalid caps received");
+    gst_object_unref (gsmdec);
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -295,7 +285,7 @@ gst_gsmdec_chain (GstPad * pad, GstBuffer * buf)
     GstBuffer *outbuf;
 
     /* always the same amount of output samples */
-    outbuf = gst_buffer_new_and_alloc (ENCODED_SAMPLES * sizeof (gsm_signal));
+    outbuf = gst_buffer_new_and_alloc (160 * sizeof (gsm_signal));
 
     /* If we are not given any timestamp, interpolate from last seen
      * timestamp (if any). */
@@ -306,13 +296,13 @@ gst_gsmdec_chain (GstPad * pad, GstBuffer * buf)
 
     /* interpolate in the next run */
     if (timestamp != GST_CLOCK_TIME_NONE)
-      gsmdec->next_ts = timestamp + gsmdec->duration;
+      gsmdec->next_ts = timestamp + (20 * GST_MSECOND);
     timestamp = GST_CLOCK_TIME_NONE;
 
-    GST_BUFFER_DURATION (outbuf) = gsmdec->duration;
+    GST_BUFFER_DURATION (outbuf) = 20 * GST_MSECOND;
     GST_BUFFER_OFFSET (outbuf) = gsmdec->next_of;
     if (gsmdec->next_of != -1)
-      gsmdec->next_of += ENCODED_SAMPLES;
+      gsmdec->next_of += 160;
     GST_BUFFER_OFFSET_END (outbuf) = gsmdec->next_of;
 
     gst_buffer_set_caps (outbuf, GST_PAD_CAPS (gsmdec->srcpad));
